// prefix header, with public (user-code-reachable) struct and typedef declarations
#include "FBtoC.h"
#include "Runtime.h"

/*
 *  Prefs.c
   conditionally included runtime support for get preferences, put preferences, kill preferences
 */


void FBPutPreferences( Str255 name, void *prefsPtr, CFIndex size )
{
	CFDataRef	dataRef = CFDataCreate( 0, prefsPtr, size );
	if ( dataRef )
	{
		CFStringRef	string = CFStringCreateWithPascalString( NULL, name, kCFStringEncodingMacRoman );
		if ( string )
		{
			CFPreferencesSetAppValue( CFSTR( "FBGetPutPreferences" ), dataRef, string );
			CFPreferencesAppSynchronize( string );
			CFRelease( string );
		}
		CFRelease( dataRef );
	}
}


void FBGetPreferences( Str255 name, void *prefsPtr )
{
	CFStringRef	string = CFStringCreateWithPascalString( NULL, name, kCFStringEncodingMacRoman );
	if ( string )
	{
		CFDataRef	dataRef = CFPreferencesCopyAppValue( CFSTR( "FBGetPutPreferences" ), string );
		if ( dataRef )
		{
			if ( CFGetTypeID( dataRef) == CFDataGetTypeID() )
			{
				CFRange	range = CFRangeMake( 0, CFDataGetLength( dataRef ) );
				CFDataGetBytes( dataRef, range, prefsPtr );
			}
			CFRelease( dataRef );
		}
		CFRelease( string );
	}
}



void FBKillPreferences( Str255 name )
{
	FSRef		folderRef, fileRef;
	CFIndex		length;
	UniChar		*chars = NULL;

	CFStringRef	string1 = CFStringCreateWithPascalString( NULL, name, kCFStringEncodingMacRoman );
	if ( string1 )
	{
		CFMutableStringRef	string2 = CFStringCreateMutableCopy( NULL, CFStringGetLength( string1 ), string1 );
		if ( string2 )
		{
			CFStringAppend( string2, CFSTR( ".plist" ) );
			if ( FSFindFolder( kOnSystemDisk, kPreferencesFolderType, kDontCreateFolder, &folderRef ) == noErr )
			{
				short		p = 0;
				FBPutPreferences( name, &p, 0 ); // null out the existing prefs file and (more importantly) the CFPreferences cache		
				length = CFStringGetLength( string2 );
				chars = malloc( length * sizeof( UniChar ) );
				CFStringGetCharacters( string2, CFRangeMake( 0, length ), chars );
				FSMakeFSRefUnicode( &folderRef, length, chars, kTextEncodingUnknown, &fileRef );
				FSDeleteObject( &fileRef );
				if ( chars != NULL ) free( chars );
			}
			CFRelease( string2 );
		}
		CFRelease( string1 );
	}
}





